/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#pragma once

#include "FDatabaseMetaDataResultSet.hxx"
#include <com/sun/star/sdbc/XDatabaseMetaData.hpp>
#include <connectivity/dbtoolsdllapi.hxx>

namespace connectivity
{
    class OOO_DLLPUBLIC_DBTOOLS OResultSetPrivileges :
        public ODatabaseMetaDataResultSet
    {
        css::uno::Reference< css::sdbc::XResultSet>   m_xTables;
        css::uno::Reference< css::sdbc::XRow>         m_xRow;
        bool                                          m_bResetValues;
    protected:
        virtual const ORowSetValue& getValue(sal_Int32 columnIndex) override;
    public:
        OResultSetPrivileges(const css::uno::Reference< css::sdbc::XDatabaseMetaData>& _rxMeta
            ,const css::uno::Any& catalog, const OUString& schemaPattern, const OUString& tableNamePattern);

        // ::comphelper::WeakComponentImplHelper
        virtual void disposing(std::unique_lock<std::mutex>&) override;
        // XResultSet
        virtual sal_Bool SAL_CALL next(  ) override;
    };
}

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
