/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#pragma once

#include <rtl/ustring.hxx>

inline constexpr OUString HID_OPTIONS_COLORCONFIG_SAVE_SCHEME
    = u"CUI_HID_OPTIONS_COLORCONFIG_SAVE_SCHEME"_ustr;
inline constexpr OUString HID_OFA_FONT_SUBST_CLB = u"CUI_HID_OFA_FONT_SUBST_CLB"_ustr;
inline constexpr OUString HID_DBPATH_CTL_PATH = u"CUI_HID_DBPATH_CTL_PATH"_ustr;
inline constexpr OUString HID_OFADLG_TREELISTBOX = u"CUI_HID_OFADLG_TREELISTBOX"_ustr;
inline constexpr OUString HID_SVX_CONFIG_TOOLBAR = u"CUI_HID_SVX_CONFIG_TOOLBAR"_ustr;
inline constexpr OUString HID_SVX_CONFIG_TOOLBAR_CONTENTS
    = u"CUI_HID_SVX_CONFIG_TOOLBAR_CONTENTS"_ustr;
inline constexpr OUString HID_SVX_CONFIG_NOTEBOOKBAR_CONTENTS
    = u"CUI_HID_SVX_CONFIG_NOTEBOOKBAR_CONTENTS"_ustr;
inline constexpr OUString HID_HANGULDLG_SUGGESTIONS_GRID
    = u"CUI_HID_HANGULDLG_SUGGESTIONS_GRID"_ustr;
inline constexpr OUString HID_HANGULDLG_SUGGESTIONS_LIST
    = u"CUI_HID_HANGULDLG_SUGGESTIONS_LIST"_ustr;
inline constexpr OUString HID_SVX_CONFIG_NAME_SUBMENU = u"CUI_HID_SVX_CONFIG_NAME_SUBMENU"_ustr;
inline constexpr OUString HID_SVX_CONFIG_RENAME_MENU = u"CUI_HID_SVX_CONFIG_RENAME_MENU"_ustr;
inline constexpr OUString HID_SVX_CONFIG_RENAME_MENU_ITEM
    = u"CUI_HID_SVX_CONFIG_RENAME_MENU_ITEM"_ustr;
inline constexpr OUString HID_SVX_CONFIG_RENAME_TOOLBAR = u"CUI_HID_SVX_CONFIG_RENAME_TOOLBAR"_ustr;
inline constexpr OUString HID_SVX_CONFIG_RENAME_TOOLBAR_ITEM
    = u"CUI_HID_SVX_CONFIG_RENAME_TOOLBAR_ITEM"_ustr;
inline constexpr OUString HID_SVX_UP_TOOLBAR_ITEM = u"CUI_HID_SVX_UP_TOOLBAR_ITEM"_ustr;
inline constexpr OUString HID_SVX_DOWN_TOOLBAR_ITEM = u"CUI_HID_SVX_DOWN_TOOLBAR_ITEM"_ustr;
inline constexpr OUString HID_SVX_SAVE_IN = u"CUI_HID_SVX_SAVE_IN"_ustr;
inline constexpr OUString HID_SVX_TOPLEVELLISTBOX = u"CUI_HID_SVX_TOPLEVELLISTBOX"_ustr;
inline constexpr OUString HID_SVX_DESCFIELD = u"CUI_HID_SVX_DESCFIELD"_ustr;
inline constexpr OUString HID_MACRO_HEADERTABLISTBOX = u"CUI_HID_MACRO_HEADERTABLISTBOX"_ustr;

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
