/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#pragma once

#include <typelib/typedescription.h>

namespace comphelper::detail
{
// This is like com::sun::star::uno::TypeDescription, but it uses TYPELIB_DANGER_GET
// (which the code used originally, but it's easier to have a class to handle ownership).
class TypeDescriptionRef
{
public:
    TypeDescriptionRef(typelib_TypeDescriptionReference* typeDef)
    {
        TYPELIB_DANGER_GET(&typeDescr, typeDef);
    }
    ~TypeDescriptionRef() { TYPELIB_DANGER_RELEASE(typeDescr); }
    typelib_TypeDescription* get() { return typeDescr; }
    typelib_TypeDescription* operator->() { return typeDescr; }
    bool is() { return typeDescr != nullptr; }
    bool equals(const TypeDescriptionRef& other) const
    {
        return typeDescr && other.typeDescr
               && typelib_typedescription_equals(typeDescr, other.typeDescr);
    }

private:
    typelib_TypeDescription* typeDescr = nullptr;
};

} // namespace

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
