/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#pragma once

#include "WTypeSelect.hxx"

class SvStream;
namespace dbaui
{
    class OCopyTableWizard;

    // Wizard Page: OWizHTMLExtend
    class OWizHTMLExtend : public OWizTypeSelect
    {
    protected:
        virtual void createReaderAndCallParser(sal_Int32 _nRows) override;
    public:
        OWizHTMLExtend(weld::Container* pPage, OCopyTableWizard* pWizard, SvStream& rStream)
            : OWizTypeSelect(pPage, pWizard, &rStream)
        {
        }

        static std::unique_ptr<OWizTypeSelect> Create(weld::Container* pPage, OCopyTableWizard* pWizard, SvStream& rInput ) { return std::make_unique<OWizHTMLExtend>(pPage, pWizard, rInput); }
    };
    // Wizard Page: OWizRTFExtend
    class OWizRTFExtend : public OWizTypeSelect
    {
    protected:
        virtual void createReaderAndCallParser(sal_Int32 _nRows) override;
    public:
        OWizRTFExtend(weld::Container* pPage, OCopyTableWizard* pWizard, SvStream& rStream)
            : OWizTypeSelect(pPage, pWizard, &rStream)
        {
        }

        static std::unique_ptr<OWizTypeSelect> Create(weld::Container* pPage, OCopyTableWizard* pWizard, SvStream& rInput) { return std::make_unique<OWizRTFExtend>(pPage, pWizard, rInput); }
    };

    // Wizard Page: OWizNormalExtend
    class OWizNormalExtend : public OWizTypeSelect
    {
    protected:
        virtual void createReaderAndCallParser(sal_Int32 _nRows) override;
    public:
        OWizNormalExtend(weld::Container* pPage, OCopyTableWizard* pWizard)
            : OWizTypeSelect(pPage, pWizard)
        {
            EnableAuto(false);
        }
    };
}

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
