/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#pragma once

#include "sbxprop.hxx"
#include <basic/sbdef.hxx>

class SbModule;

class SbProperty final : public SbxProperty
{
    friend class SbiFactory;
    friend class SbModule;
    friend class SbProcedureProperty;
    SbModule* pMod;
    SbProperty( const OUString&, SbxDataType, SbModule* );
    virtual ~SbProperty() override;
public:
    SBX_DECL_PERSIST_NODATA(SBXID_BASICPROP,1);
    SbModule* GetModule() { return pMod; }
};

typedef tools::SvRef<SbProperty> SbPropertyRef;

class SbProcedureProperty final : public SbxProperty
{
    bool mbSet;     // Flag for set command

    virtual ~SbProcedureProperty() override;

public:
    SbProcedureProperty( const OUString& r, SbxDataType t )
            : SbxProperty( r, t ) // , pMod( p )
            , mbSet( false )
    {}

    bool isSet() const
        { return mbSet; }
    void setSet( bool bSet )
        { mbSet = bSet; }
};

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
