/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4; fill-column: 100 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#pragma once

#include <rtl/ustring.hxx>

// Dialog Controls ---------------------------------------------------------------

inline constexpr OUString RID_STR_CLASS_CONTROL = u"Control"_ustr;
inline constexpr OUString RID_STR_CLASS_DIALOG = u"Dialog"_ustr;
inline constexpr OUString RID_STR_CLASS_BUTTON = u"CommandButton"_ustr;
inline constexpr OUString RID_STR_CLASS_RADIOBUTTON = u"OptionButton"_ustr;
inline constexpr OUString RID_STR_CLASS_CHECKBOX = u"CheckBox"_ustr;
inline constexpr OUString RID_STR_CLASS_LISTBOX = u"ListBox"_ustr;
inline constexpr OUString RID_STR_CLASS_COMBOBOX = u"ComboBox"_ustr;
inline constexpr OUString RID_STR_CLASS_GROUPBOX = u"FrameControl"_ustr;
inline constexpr OUString RID_STR_CLASS_EDIT = u"TextField"_ustr;
inline constexpr OUString RID_STR_CLASS_FIXEDTEXT = u"Label"_ustr;
inline constexpr OUString RID_STR_CLASS_IMAGECONTROL = u"ImageControl"_ustr;
inline constexpr OUString RID_STR_CLASS_PROGRESSBAR = u"ProgressBar"_ustr;
inline constexpr OUString RID_STR_CLASS_SCROLLBAR = u"ScrollBar"_ustr;
inline constexpr OUString RID_STR_CLASS_FIXEDLINE = u"FixedLine"_ustr;
inline constexpr OUString RID_STR_CLASS_DATEFIELD = u"DateField"_ustr;
inline constexpr OUString RID_STR_CLASS_TIMEFIELD = u"TimeField"_ustr;
inline constexpr OUString RID_STR_CLASS_NUMERICFIELD = u"NumericField"_ustr;
inline constexpr OUString RID_STR_CLASS_CURRENCYFIELD = u"CurrencyField"_ustr;
inline constexpr OUString RID_STR_CLASS_FORMATTEDFIELD = u"FormattedField"_ustr;
inline constexpr OUString RID_STR_CLASS_PATTERNFIELD = u"PatternField"_ustr;
inline constexpr OUString RID_STR_CLASS_FILECONTROL = u"FileControl"_ustr;
inline constexpr OUString RID_STR_CLASS_TREECONTROL = u"TreeControl"_ustr;
inline constexpr OUString RID_STR_CLASS_GRIDCONTROL = u"GridControl"_ustr;
inline constexpr OUString RID_STR_CLASS_SPINCONTROL = u"SpinButton"_ustr;
inline constexpr OUString RID_STR_CLASS_HYPERLINKCONTROL = u"HyperlinkControl"_ustr;

/* vim:set shiftwidth=4 softtabstop=4 expandtab cinoptions=b1,g0,N-s cinkeys+=0=break: */
