/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#pragma once
#include <sfx2/tabdlg.hxx>
#include <rtl/ustring.hxx>

class OptLanguageToolTabPage : public SfxTabPage
{
public:
    OptLanguageToolTabPage(weld::Container* pPage, weld::DialogController* pController,
                           const SfxItemSet& rSet);
    virtual ~OptLanguageToolTabPage() override;
    static std::unique_ptr<SfxTabPage>
    Create(weld::Container* pPage, weld::DialogController* pController, const SfxItemSet* rAttrSet);

    virtual OUString GetAllStrings() override;

    virtual bool FillItemSet(SfxItemSet* rSet) override;
    virtual void Reset(const SfxItemSet* rSet) override;

private:
    std::unique_ptr<weld::Entry> m_xBaseURLED;
    std::unique_ptr<weld::Widget> m_xBaseURLImg;
    std::unique_ptr<weld::Entry> m_xUsernameED;
    std::unique_ptr<weld::Widget> m_xUsernameImg;
    std::unique_ptr<weld::Entry> m_xApiKeyED;
    std::unique_ptr<weld::Widget> m_xApiKeyImg;
    std::unique_ptr<weld::Entry> m_xRestProtocol;
    std::unique_ptr<weld::Widget> m_xRestProtocolImg;
    std::unique_ptr<weld::CheckButton> m_xActivateBox;
    std::unique_ptr<weld::Widget> m_xActivateBoxImg;
    std::unique_ptr<weld::CheckButton> m_xSSLDisableVerificationBox;
    std::unique_ptr<weld::Widget> m_xSSLDisableVerificationBoxImg;
    std::unique_ptr<weld::Frame> m_xApiSettingsFrame;

    void EnableControls(bool bEnable);

    DECL_LINK(CheckHdl, weld::Toggleable&, void);
};
