/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#pragma once

#include "MultipleItemConverter.hxx"
#include <com/sun/star/awt/Size.hpp>
#include <rtl/ref.hxx>
#include <optional>

namespace com::sun::star::lang { class XMultiServiceFactory; }
namespace chart { class ChartModel; }
class SdrModel;

namespace chart::wrapper {

class AllAxisItemConverter final : public MultipleItemConverter
{
public:
    AllAxisItemConverter(
        const rtl::Reference<::chart::ChartModel> & xChartModel,
        SfxItemPool& rItemPool,
        SdrModel& rDrawModel,
        const std::optional<css::awt::Size>& pRefSize );

    virtual ~AllAxisItemConverter() override;

protected:
    virtual const WhichRangesContainer& GetWhichPairs() const override;
};

class AllGridItemConverter final : public MultipleItemConverter
{
public:
    AllGridItemConverter(
        const rtl::Reference<::chart::ChartModel>& xChartModel,
        SfxItemPool& rItemPool,
        SdrModel& rDrawModel,
        const css::uno::Reference<css::lang::XMultiServiceFactory> & xNamedPropertyContainerFactory );
    virtual ~AllGridItemConverter() override;

protected:
    virtual const WhichRangesContainer& GetWhichPairs() const override;
};

class AllDataLabelItemConverter final : public MultipleItemConverter
{
public:
    AllDataLabelItemConverter(
        const rtl::Reference<::chart::ChartModel>& xChartModel,
        SfxItemPool& rItemPool,
        SdrModel& rDrawModel,
        const css::uno::Reference<css::lang::XMultiServiceFactory>& xNamedPropertyContainerFactory );

    virtual ~AllDataLabelItemConverter() override;

protected:
    virtual const WhichRangesContainer& GetWhichPairs() const override;
};

class AllTitleItemConverter final : public MultipleItemConverter
{
public:
    AllTitleItemConverter(
        const rtl::Reference<::chart::ChartModel>& xChartModel,
        SfxItemPool& rItemPool, SdrModel& rDrawModel,
        const css::uno::Reference<css::lang::XMultiServiceFactory>& xNamedPropertyContainerFactory );

    virtual ~AllTitleItemConverter() override;

protected:
    virtual const WhichRangesContainer& GetWhichPairs() const override;
};

class AllSeriesStatisticsConverter final : public MultipleItemConverter
{
public:
    AllSeriesStatisticsConverter(
        const rtl::Reference<::chart::ChartModel>& xChartModel, SfxItemPool& rItemPool );
    virtual ~AllSeriesStatisticsConverter() override;

protected:
    virtual const WhichRangesContainer& GetWhichPairs() const override;
};

} // namespace chart::wrapper

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
