/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#include "xmlTableFilterPattern.hxx"
#include "xmlTableFilterList.hxx"

namespace dbaxml
{
OXMLTableFilterPattern::OXMLTableFilterPattern( SvXMLImport& rImport
                ,bool _bNameFilter
                ,OXMLTableFilterList& _rParent)
    :SvXMLImportContext( rImport )
    ,m_rParent(_rParent)
    ,m_bNameFilter(_bNameFilter)
{

}

OXMLTableFilterPattern::~OXMLTableFilterPattern()
{

}

void OXMLTableFilterPattern::characters( const OUString& rChars )
{
    maBuffer.append(rChars);
}

void OXMLTableFilterPattern::endFastElement( sal_Int32 )
{
    OUString sChars = maBuffer.makeStringAndClear();
    if ( m_bNameFilter )
        m_rParent.pushTableFilterPattern(sChars);
    else
        m_rParent.pushTableTypeFilter(sChars);
}

} // namespace dbaxml

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
