/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#pragma once

#include <rtl/ustring.hxx>

inline constexpr OUString CHART_UNONAME_SORT_BY_XVALUES = u"SortByXValues"_ustr;
inline constexpr OUString CHART_UNONAME_SPLINE_TYPE = u"SplineType"_ustr;
inline constexpr OUString CHART_UNONAME_SPLINE_ORDER = u"SplineOrder"_ustr;
inline constexpr OUString CHART_UNONAME_SPLINE_RESOLUTION = u"SplineResolution"_ustr;
inline constexpr OUString CHART_UNONAME_CURVE_STYLE = u"CurveStyle"_ustr;
inline constexpr OUString CHART_UNONAME_CURVE_RESOLUTION = u"CurveResolution"_ustr;
inline constexpr OUString CHART_UNONAME_NUMFMT = u"NumberFormat"_ustr;
inline constexpr OUString CHART_UNONAME_LINK_TO_SRC_NUMFMT = u"LinkNumberFormatToSource"_ustr;
inline constexpr OUString CHART_UNONAME_ERRORBAR_X = u"ErrorBarX"_ustr;
inline constexpr OUString CHART_UNONAME_ERRORBAR_Y = u"ErrorBarY"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL = u"Label"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_SEP = u"LabelSeparator"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_BORDER_STYLE = u"LabelBorderStyle"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_BORDER_WIDTH = u"LabelBorderWidth"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_BORDER_COLOR = u"LabelBorderColor"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_BORDER_DASH = u"LabelBorderDash"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_BORDER_DASHNAME = u"LabelBorderDashName"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_BORDER_TRANS = u"LabelBorderTransparency"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_FILL_STYLE = u"LabelFillStyle"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_FILL_BACKGROUND = u"LabelFillBackground"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_FILL_HATCH_NAME = u"LabelFillHatchName"_ustr;
inline constexpr OUString CHART_UNONAME_LABEL_FILL_COLOR = u"LabelFillColor"_ustr;
inline constexpr OUString CHART_UNONAME_CUSTOM_LABEL_FIELDS = u"CustomLabelFields"_ustr;

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
