/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#pragma once

#include <component/CTable.hxx>

namespace com::sun::star::text
{
class XTextTable;
}

namespace connectivity::writer
{
using OWriterTable_BASE = component::OComponentTable;
class OWriterConnection;

class OWriterTable : public OWriterTable_BASE
{
private:
    css::uno::Reference<css::text::XTextTable> m_xTable;
    OWriterConnection* m_pWriterConnection;
    sal_Int32 m_nStartCol = 0;
    sal_Int32 m_nDataCols = 0;
    bool m_bHasHeaders = false;

    void fillColumns();

public:
    OWriterTable(sdbcx::OCollection* _pTables, OWriterConnection* _pConnection,
                 const OUString& Name, const OUString& Type);

    bool fetchRow(OValueRefRow& _rRow, const OSQLColumns& _rCols, bool bRetrieveData) override;

    void SAL_CALL disposing() override;

    void construct() override;
};

} // namespace connectivity::writer

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
