/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#include <FormattedStringHelper.hxx>
#include <FormattedString.hxx>
#include <comphelper/diagnose_ex.hxx>
#include <comphelper/property.hxx>

namespace chart
{

using namespace ::com::sun::star;
using namespace ::com::sun::star::chart2;
using ::com::sun::star::uno::Reference;

rtl::Reference< ::chart::FormattedString >
            FormattedStringHelper::createFormattedString(
                      const OUString & rString
                    , const Reference< beans::XPropertySet > & xTextProperties ) noexcept
{
    rtl::Reference< FormattedString > xFormStr;
    try
    {
        xFormStr = new FormattedString();

        xFormStr->setString( rString );

        // set character properties
        comphelper::copyProperties(
            xTextProperties, Reference< beans::XPropertySet >( static_cast<cppu::OWeakObject*>(xFormStr.get()), uno::UNO_QUERY_THROW ) );
    }
    catch( const uno::Exception & )
    {
        DBG_UNHANDLED_EXCEPTION("chart2");
    }

    return xFormStr;
}

} //namespace chart

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
