/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4; fill-column: 100 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#pragma once

#include <rtl/string.hxx>
#include <codemaker/options.hxx>

class NetOptions : public Options
{
public:
    NetOptions()
    {
        m_program = "netmaker"_ostr;
        m_version = "0.1.0"_ostr;
    }

    bool initOptions(int argc, char* argv[], bool bCmdFile = false) override;

    OString prepareHelp() override;
    OString prepareVersion() const;

private:
    OString m_version;
};

/* vim:set shiftwidth=4 softtabstop=4 expandtab cinoptions=b1,g0,N-s cinkeys+=0=break: */
